package com.gic.haoban.manage.web.controller;

import cn.hutool.crypto.SecureUtil;
import cn.hutool.crypto.symmetric.AES;
import com.alibaba.dubbo.common.URL;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gic.commons.util.ToolUtil;
import com.gic.haoban.base.api.common.pojo.dto.WebLoginDTO;
import com.gic.haoban.common.anno.IgnoreLogin;
import com.gic.haoban.common.utils.AuthWebRequestUtil;
import com.gic.haoban.common.utils.EntityUtil;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.manage.api.dto.LoginDTO;
import com.gic.haoban.manage.api.dto.StaffDTO;
import com.gic.haoban.manage.api.dto.WxApplicationDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseDTO;
import com.gic.haoban.manage.api.service.StaffApiService;
import com.gic.haoban.manage.api.service.WxApplicationApiService;
import com.gic.haoban.manage.api.service.WxEnterpriseApiService;
import com.gic.haoban.manage.api.service.WxEnterpriseRelatedApiService;
import com.gic.haoban.manage.web.config.Config;
import com.gic.haoban.manage.web.errCode.HaoBanErrCode;
import com.gic.redis.data.util.RedisUtil;
import com.gic.wechat.api.dto.qywx.UserDTO;
import com.gic.wechat.api.service.qywx.QywxCorpApiService;
import com.gic.wechat.api.service.qywx.QywxSuiteApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.view.RedirectView;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;


@RestController
public class LoginController extends WebBaseController {

    private static final Logger logger = LoggerFactory.getLogger(LoginController.class);

    private static final String LOGIN_KEY = "haoban#login$key";

    @Autowired
    private StaffApiService staffApiService;

    @Autowired
    private QywxCorpApiService qywxCorpApiService;

    @Autowired
    private WxEnterpriseApiService wxEnterpriseApiService;

    @Autowired
    private WxEnterpriseRelatedApiService wxEnterpriseRelatedApiService;

    @Autowired
    private QywxSuiteApiService qywxSuiteApiService;

    @Autowired
    private WxApplicationApiService wxApplicationApiService;

    @Autowired
    private Config config;

    @RequestMapping("login")
    @Deprecated
    public void login(String auth_code, HttpServletResponse response) {
        String userJson = qywxCorpApiService.getLoginUserInfo(config.getCorpid(), auth_code);
        if (StringUtils.isBlank(userJson)) {
            return;
        }

        UserDTO user = JSON.parseObject(userJson, UserDTO.class);
        String corpId = user.getCorpid();
        logger.info("【企业微信登录】userId={},auth_code={},corpId={}", user.getUserid(), auth_code, corpId);
        WxEnterpriseDTO enterprise = wxEnterpriseApiService.getEnterpriseBycorpId(corpId);
        logger.info("【企业微信】enterprise={}", JSON.toJSONString(enterprise));
        StaffDTO loginStaff = null;
        if (enterprise != null) {
            loginStaff = staffApiService.selectByUserIdAndEnterpriseId(user.getUserid(), enterprise.getWxEnterpriseId());
        }

        if (loginStaff == null) {
            if (enterprise != null) {
                staffApiService.wxFristAdd(JSON.toJSONString(user), enterprise.getWxEnterpriseId());
                loginStaff = staffApiService.selectByUserIdAndEnterpriseId(user.getUserid(), enterprise.getWxEnterpriseId());
            }
        }

        LoginDTO loginVO = new LoginDTO();
        logger.info("【好办web登录】staff={}", JSON.toJSONString(loginStaff));
        if (loginStaff == null) {
            return;
        }
        loginVO.setWxEnterpriseId(loginStaff.getWxEnterpriseId());
        loginVO.setStaffDTO(loginStaff);
        WebLoginDTO webLoginDTO = EntityUtil.changeEntityByOrika(WebLoginDTO.class, loginVO);
        AuthWebRequestUtil.setSessionUser(webLoginDTO);
        String redirectUri = config.getHost() + "index";

        redirectUri = redirectUri.replace("haoban-manage3-web", "haoban-3/#");
        String token = ToolUtil.randomUUID() + "-" + ToolUtil.randomStringByTime();
        AuthWebRequestUtil.setAppLoginUser(token, webLoginDTO);
        try {
            Cookie cookie = new Cookie("token", token);
            cookie.setPath("/");
            response.addCookie(cookie);
            response.sendRedirect(redirectUri);
            response.setHeader("token", token);
        } catch (IOException e) {
            logger.warn("异常", e);
        }
    }

    @RequestMapping("login_local")
    @Deprecated
    public HaobanResponse login() {
        LoginDTO loginVO = new LoginDTO();
        StaffDTO staff = staffApiService.selectById("2205f92cdb5f468cb996e91dded414c2");
        String token = ToolUtil.randomUUID() + "-" + ToolUtil.randomStringByTime();
        logger.info("【好办登录】staff={}", JSON.toJSONString(staff));
        loginVO.setWxEnterpriseId(staff.getWxEnterpriseId());
        loginVO.setStaffDTO(staff);
        AuthWebRequestUtil.setAppLoginUser(token, EntityUtil.changeEntityByOrika(WebLoginDTO.class,loginVO));
        Cookie cookie = new Cookie("token", token);
        cookie.setPath("/");
        return resultResponse(HaoBanErrCode.ERR_1, loginVO);
    }

    /**
     * 获取登录信息
     *
     * @return res
     */
    @RequestMapping("get-login-info")
    public HaobanResponse getLoginInfo() {
        return resultResponse(HaoBanErrCode.ERR_1, AuthWebRequestUtil.getLoginUser());
    }

    /**
     * 好办-扫码登录
     * @return
     * @throws UnsupportedEncodingException
     */
    @RequestMapping("get-login-qrcode")
    public HaobanResponse getLoginQrcode() throws UnsupportedEncodingException {
        String corpid = config.getCorpid();
        String redirectUri = config.getHost() + "login";
        String url = "https://open.work.weixin.qq.com/wwopen/sso/3rd_qrConnect?appid=" + corpid + "&redirect_uri=" + URLEncoder.encode(redirectUri, "UTF-8") + "&state=web_login@gyoss9&usertype=admin";
        return resultResponse(HaoBanErrCode.ERR_1, URL.decode(url));
    }

    /**
     * 运维登录
     *
     * @param wxEnterpriseId
     * @param random
     * @param response
     * @return
     */
    @IgnoreLogin
    @RequestMapping("yw-login")
    public HaobanResponse ywLogin(String wxEnterpriseId, String random, HttpServletResponse response) {
        //判断是否存在random
        Object o = RedisUtil.getCache(random);
        if (o == null) {
            return resultResponse(HaoBanErrCode.ERR_4);
        }
        StaffDTO staff = staffApiService.getAndAddSupperAdmin(wxEnterpriseId);
        if (staff == null) {
            logger.info("运维登录超级管理员不存在");
            return resultResponse(HaoBanErrCode.ERR_6);
        }
        LoginDTO loginVO = new LoginDTO();
        loginVO.setWxEnterpriseId(wxEnterpriseId);
        loginVO.setStaffDTO(staff);
        WebLoginDTO webLoginDTO = EntityUtil.changeEntityByOrika(WebLoginDTO.class, loginVO);
        AuthWebRequestUtil.setSessionUser(webLoginDTO);
        String redirectUri = config.getHost() + "index";
        redirectUri = redirectUri.replace("haoban-manage3-web", "haoban-3/#");
        String token = ToolUtil.randomUUID() + "-" + ToolUtil.randomStringByTime();
        AuthWebRequestUtil.setAppLoginUser(token, webLoginDTO);
        try {
            Cookie cookie = new Cookie("token", token);
            cookie.setPath("/");
            response.addCookie(cookie);
            response.sendRedirect(redirectUri);
            response.setHeader("token", token);
        } catch (IOException e) {
          logger.error("好办登录失败",e);
        }
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    /**
     * gic登录
     *
     * @param random
     * @param wxEnterpriseId
     * @param response
     * @return
     */
    @IgnoreLogin
    @RequestMapping("gic-login")
    public ModelAndView gicLogin(String random, String wxEnterpriseId, HttpServletResponse response) {
        //判断是否存在random
        AES aes = SecureUtil.aes(LOGIN_KEY.getBytes());
        String o = aes.decryptStr(random);
        if (o == null || StringUtils.isBlank(wxEnterpriseId)) {
            logger.info("解密异常");
            ModelAndView model = new ModelAndView(new RedirectView("/haoban-3/#/gic-error", false, false));
            return model;
        }
        JSONObject jsonObject = JSONObject.parseObject(o);
        String gicEnterpriseId = jsonObject.getString("enterpriseId");
        String loginPhoneNumber = jsonObject.getString("phoneNumber");
        List<WxEnterpriseDTO> enterpriseDTOList = wxEnterpriseRelatedApiService.listWxEnterpriseByEid(gicEnterpriseId);
        if (CollectionUtils.isEmpty(enterpriseDTOList)) {
            logger.info("登录失败，不存在对应企业:{}", o);
            ModelAndView model = new ModelAndView(new RedirectView("/haoban-3/#/gic-error", false, false));
            return model;
        }

        WxEnterpriseDTO detailDTO = enterpriseDTOList.stream().filter(dto -> dto.getWxEnterpriseId().equals(wxEnterpriseId)).findFirst().orElse(null);

        if (null == detailDTO) {
            logger.info("登录失败，不存在对应企业:{}", o);
            ModelAndView model = new ModelAndView(new RedirectView("/haoban-3/#/gic-error", false, false));
            return model;
        }

        StaffDTO staff = staffApiService.selectByNationcodeAndPhoneNumber(detailDTO.getWxEnterpriseId(), "86", loginPhoneNumber);
        if (null == staff) {
            logger.info("登录用户手机号不存在!{}", o);
            ModelAndView model = new ModelAndView(new RedirectView("/haoban-3/#/gic-error", false, false));
            return model;
        }

        WxApplicationDTO application = wxApplicationApiService.selectByWxEnterpriseIdAndApplicationType(wxEnterpriseId, 2);
        if (null == application) {
            logger.info("登录企业应用 没授权!{}");
            ModelAndView model = new ModelAndView(new RedirectView("/haoban-3/#/gic-error", false, false));
            return model;
        }
        WxEnterpriseDTO enterprise = wxEnterpriseApiService.getOne(wxEnterpriseId);
        if (null == enterprise) {
            logger.info("登录失败，不存在对应企业:{}", o);
            ModelAndView model = new ModelAndView(new RedirectView("/haoban-3/#/gic-error", false, false));
            return model;
        }
        //获取管理员列表
        String corpid = enterprise.getCorpid();
        String adminList = qywxSuiteApiService.getAdminList(corpid, application.getSiteId(), Integer.parseInt(application.getAgentId()));
        List<String> userIds = new ArrayList<String>();
        if (StringUtils.isNotBlank(adminList)) {
            JSONArray jsonArr = JSON.parseArray(adminList);
            for (Object object : jsonArr) {
                JSONObject json = JSON.parseObject(JSON.toJSONString(object));
                userIds.add(json.getString("userid"));
            }
        }
        //兼容
        if (corpid.length() > 20) {
            if (!userIds.contains(staff.getWxOpenUseId())) {
                logger.info("登录失败，当前用户不是管理员:{}", JSON.toJSONString(userIds));
                ModelAndView model = new ModelAndView(new RedirectView("/haoban-3/#/gic-error", false, false));
                return model;
            }
        } else {
            if (!userIds.contains(staff.getWxUserId())) {
                logger.info("登录失败，当前用户不是管理员:{}", JSON.toJSONString(userIds));
                ModelAndView model = new ModelAndView(new RedirectView("/haoban-3/#/gic-error", false, false));
                return model;
            }
        }
        LoginDTO loginDTO = new LoginDTO();
        loginDTO.setWxEnterpriseId(detailDTO.getWxEnterpriseId());
        loginDTO.setStaffDTO(staff);
        String redirectUri = config.getHost() + "index";
        redirectUri = redirectUri.replace("haoban-manage3-web", "haoban-3/#");
        String token = ToolUtil.randomUUID() + "-" + ToolUtil.randomStringByTime();
        WebLoginDTO webLoginDTO = EntityUtil.changeEntityByOrika(WebLoginDTO.class, loginDTO);
        AuthWebRequestUtil.setSessionUser(webLoginDTO);
        AuthWebRequestUtil.setAppLoginUser(token, webLoginDTO);
        try {
            Cookie cookie = new Cookie("token", token);
            cookie.setPath("/");
            response.addCookie(cookie);
            response.sendRedirect(redirectUri);
            response.setHeader("token", token);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 登录微信企业列表
     * @param random
     * @return
     */
    @RequestMapping("login-wx-enterprise-list")
    public HaobanResponse loginWxEnterpriseList(String random) {
        //判断是否存在random
        AES aes = SecureUtil.aes(LOGIN_KEY.getBytes());
        String o = aes.decryptStr(random);
        if (o == null) {
            logger.info("解密异常");
            return resultResponse(HaoBanErrCode.ERR_5);
        }
        logger.info(o);
        JSONObject jsonObject = JSONObject.parseObject(o);
        String gicEnterpriseId = jsonObject.getString("enterpriseId");
        String wxEnterpriseId = jsonObject.getString("wxEnterpriseId");
        List<WxEnterpriseDTO> enterpriseDTOList = wxEnterpriseRelatedApiService.listWxEnterpriseByEid(gicEnterpriseId);
        if (CollectionUtils.isEmpty(enterpriseDTOList)) {
            logger.info("登录失败，不存在对应企业:{}", o);
            return resultResponse(HaoBanErrCode.ERR_6);
        }
        if (StringUtils.isNotBlank(wxEnterpriseId)) {
            List<WxEnterpriseDTO> collect = enterpriseDTOList.stream().filter(dto -> dto.getWxEnterpriseId().equals(wxEnterpriseId)).collect(Collectors.toList());
            return resultResponse(HaoBanErrCode.ERR_1, collect);
        }
        return resultResponse(HaoBanErrCode.ERR_1, enterpriseDTOList);
    }

    /**
     * 登出
     * @return
     */
    @RequestMapping("login-out")
    public HaobanResponse loginOut() {
        String token = "";
        Cookie[] cookies = AuthWebRequestUtil.getRequest().getCookies();
        if (cookies != null) {
            for (Cookie cookie : cookies) {
                if (cookie.getName().equals("token")) {
                    token = cookie.getValue();
                    break;
                }
            }
        }
        if (StringUtils.isNotBlank(token)) {
            RedisUtil.delCache(token);
        }
        return resultResponse(HaoBanErrCode.ERR_1);

    }

}
